#!/usr/bin/env python3
'''
Created on Jan 11, 2019

@author: bohle
'''

descr='''
     __________________________________________________
    |                                                  |
    |                    NMRPLOT                       |
    |          Plotting of NMR spectral data           |
    |             University of Bonn, MCTC             |
    |                 January 2019                     |
    |                     v 1.01                       |
    |                   F. Bohle                       |
    |__________________________________________________|
    '''

useit='''
    End     Endremove    Startremove                 Start
    +               +    +                               +
    +---------------+----+-------------------------------+
    lower field                               higher field
                        delta /ppm
    '''

def checkval(value):
    ''' check if value larger than 0.0 or smaller than 1.0'''
    x = float(value)
    if x < 0  or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 1.0]"%(x,))
    return x

def cml(descr):
    """ Get args object from commandline interface.
        Needs argparse module."""

    parser = argparse.ArgumentParser(description='',formatter_class=argparse.ArgumentDefaultsHelpFormatter,usage=argparse.SUPPRESS) #argparse.RawDescriptionHelpFormatter) #, 
    parser.add_argument("-start", "--startppm",
                        dest="start",
                        action="store",
                        required=False,
                        default=0,
                        type=float,
                        help="Start plotting from '<start>' ppm.")
    parser.add_argument("-end", "--endppm",
                        dest="end",
                        action="store",
                        required=False,
                        default=11,
                        type=float,
                        help="End plotting at '<end>' ppm. Value of end has to be larger than value of start.")
    parser.add_argument("-startremove", "--startremove",
                        dest="startremove",
                        action="store",
                        required=False,
                        default=None,
                        type=float,
                        help="Start cutting from spectrum at '<startremove>' ppm.")
    parser.add_argument("-endremove", "--endremove",
                        dest="endremove",
                        action="store",
                        required=False,
                        default=None,
                        type=float,
                        help="End cutting from spectrum at '<endremove>' ppm. Value of endremove has to be larger than value of startremove.")
    parser.add_argument("-title", "--title",
                        dest="title",
                        action="store",
                        required=False,
                        default='NMR-PLOT',
                        type=str,
                        help="Set title of entire plot. If no title is required use '<--title ''>'.")
    parser.add_argument("-lw", "--linewidth",
                        dest="linewidth",
                        action="store",
                        required=False,
                        default=0.8,
                        type=float,
                        help="Set linewidth.")
    parser.add_argument('-i',"--input",
                        dest='file',
                        action='store',
                        required=True,
                        nargs='+',
                        help='Provide input_file(s) [max 3 files] -i input1(theory1) input2(theory2) input3(experiment/predicition); inputfiles format is two columns: column1 ppm , column2 intensity; if several files are provided the last one will be inverted')
    parser.add_argument('-l',"--label",
                        dest='label',
                        action='store',
                        required=False,
                        default=[],
                        nargs='+',
                        help='Provide labels for all files provided [max 3 files] -l label1 label2 label3, if no labels are provided, filename is used as label')
    parser.add_argument("-fontsize", "--fontsize",
                        dest="fontsize",
                        action="store",
                        required=False,
                        default=15,
                        type=float,
                        help="Set fontsize for entire plot.")
    parser.add_argument("-keybox", "--keybox",
                        dest="keybox",
                        action="store_true",
                        required=False,
                        default=False,
                        help="Set Frame around key.")
    parser.add_argument("-ontop", "--ontop",
                        dest="ontop",
                        action="store_true",
                        required=False,
                        default=False,
                        help="Plot all spectra ontop of each other.")
    parser.add_argument("-c", "--colors",
                        dest="colors",
                        action="store",
                        nargs='+',
                        choices=['gray','blue','cyan','red','green','magenta','yellow','black'],
                        required=False,
                        metavar='',
                        default=['blue','cyan','red'],
                        help="Select colors. Possible are: {}".format(['gray','blue','cyan','red','green','magenta','yellow','black']))
    args_defaults = {'colors': parser.get_default('colors')}
    parser.add_argument("-cut", "--cut",
                        dest="cut",
                        action="store",
                        nargs='+',
                        required=False,
                        type=checkval, ### own function
                        default=[1.0,1.0,1.0],
                        help="Cut intensity. Accepts values from 0.0 (flat line) to 1.0 (full intensity).")
    args_defaults['cut'] = parser.get_default('cut')
    parser.add_argument("-o", "--output",
                        dest="out",
                        action="store",
                        required=False,
                        default='nmrplot',
                        help="Provide name of the output file without fileending.")
    parser.add_argument("--debug",
                        dest="debug",
                        action="store_true",
                        required=False,
                        default=False,
                        help=argparse.SUPPRESS)
    parser.add_argument("-s", "--shift",
                        dest="shift",
                        action="store",
                        nargs='+',
                        type=float,
                        required=False,
                        default=[],
                        help="Shift ppm of each inputfile separately using: --shift float float float, e.g. --shift 10.0 0.0 -5.0, each file needs its own value")
    
    args = parser.parse_args()
    return args, args_defaults

def readinput(filename, ppm ,intensit, number):
    ''' read input from given filename into x and y'''
    if len(args.file) == 3:
        if number in [0,1]:
            yvalue=1
        else:
            yvalue=-1
    elif len(args.file) == 2:
        if number == 0:
            yvalue=1
        else:
            yvalue=-1
    else:
        yvalue=1
    with open(filename, 'r') as inp:
        data = inp.readlines()
    x = []
    y = []
    for line in data:
        x.append(float(line.split()[0]))
        y.append(float(line.split()[1])*yvalue)
    ppm.append(x)
    intensit.append(y)
    return ppm, intensit

def axdefaultsettings(ax):
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.tick_params(axis='x',which='both',bottom=False,top=False,labelbottom=False, labelsize=args.fontsize)
    ax.tick_params(axis='y',which='both',left=False,right=False,labelleft=False)
    ax.get_yaxis().set_visible(False)
    ax.set_ymargin(0.002) ###(matplotlib 2.x space between spine and axis Set padding of Y data limits prior to autoscaling.
    ax.autoscale(enable=True, axis='y', tight=True)
    return

def equal_ticks(axes1, axes2):
    xticksax1 = (abs(axes1.get_xticks()[0]-axes1.get_xticks()[1]))
    xticksax2 = (abs(axes2.get_xticks()[0]-axes2.get_xticks()[1]))
    #print( axes1.get_xticks() )
    #print( axes2.get_xticks() )
    #print(xticksax1)
    #print(xticksax2)

    #xticksax1 = 2
    #xticksax2 = 2

    if round(xticksax1,2) >= round(xticksax2,2):
        nex = args.startremove + xticksax1
        while True:
            if nex > args.endremove:
                break
            nex += xticksax1
        t1 = np.arange(nex, args.end, step=xticksax1)
        if (abs(t1[0]) - abs(args.endremove)) < round(xticksax1/2,2) :
            t1 = t1[1:]
        t2 = np.arange(args.start,args.startremove, step=xticksax1)
        axes2.set_xticks(t2)
        axes1.set_xticks(t1)
    else:
        nex = args.startremove + xticksax2
        while True:
            if nex > args.endremove:
                break
            nex += xticksax2
        t1 = np.arange(nex, args.end, step=xticksax2)
        if (abs(t1[0]) - abs(args.endremove)) < round(xticksax2/2,2) :
            t1 = t1[1:]
        t2 = np.arange(args.start,args.startremove, step=xticksax2)
        axes2.set_xticks(t2)
        axes1.set_xticks(t1)

    return axes1, axes2

def debug():
    print('    Current python version: {}'.format(version_info))
    print('    {}'.format(plt.__file__))
    print('    {}'.format(gridspec.__file__))
    print('    {}'.format(np.__file__))
    exit(0)


if __name__ == '__main__':
    ##try and except handling of imports
    try:
        import argparse
    except ImportError:
        raise ImportError('Error while importing argparse module!')
    import matplotlib.pyplot as plt
    from matplotlib import gridspec
    import numpy as np
    from sys import version_info
    from sys import exit
    
    print(descr)  ### Program description
    args, args_defaults = cml(descr)
    
    if args.debug:
        debug()
    
    ### errors from start end startremove endremove
    errors = []
    if not args.start < args.end:
        errors.append('    Error! Start has to be smaller than end! Going to exit!')
    if args.startremove is not None or args.endremove is not None:
        if args.startremove is None or args.endremove is None:
            errors.append('    Error! Both startremove and endremove have to be set!') 
        else:
            if args.startremove is not None or args.endremove is not None: 
                if not (args.startremove < args.endremove):
                    errors.append('    Error! Startremove has to be smaller than endremove! Going to exit!')
                if args.start >= args.startremove:
                    errors.append('    Error! Startremove has to be larger than start! Going to exit!')
                if args.end <= args.endremove:
                    errors.append('    Error! Endremove has to be smaller than end! Going to exit!')
    if len(errors) >=1:
        for item in errors:
            print(item)
        print(useit)
        exit(1)

    if len(args.label) < len(args.file):  ### labels from filenames
        print('    Labels not provided or not provided for every file, using filenames as labels.')
        args.label = []
        for file in args.file:
            if file.split('.')[0] is not '':
                args.label.append(file.split('.')[0])
            else:
                args.label.append(file.split('.')[1])

    if len(args.colors) < len(args.file): ### colors
        print('    Provided less colors than files, choosing defaults: {}.'.format(args_defaults['colors']))
        args.colors = args_defaults['colors']

    if len(args.file) == 1:
        print('    Plotting only 1 data file.')
    else:
        print('    Plotting {} data files.'.format(len(args.file)))

    ### Get data from data files
    ppm = []
    intensit = []
    i = 0
    for file in args.file:
        try:
            ppm, intensit = readinput(file,ppm,intensit,i)
        except IOError:
            print('    File: {} does not exist! Terminating now!'.format(file))
            exit(1)
        i+=1

    ### ppm ranges of datafiles:
    for i in range(len(args.file)):
        print('    ppm range goes from {:6.2f} to {:6.2f} in file: {}.'.format(min(ppm[i]),max(ppm[i]),args.file[i]))

    ### cut  in case of too large intensities
    if len(args.cut) < len(args.file):
        print('    Provided less cut values than files, therefore chosing defaults.')
        args.cut = args_defaults['cut']
    for i in range(len(args.file)):
        if i+1 <len(args.file):
            newmax= max(intensit[i]) * args.cut[i]
            intensit[i] = [newmax if x >= newmax else x for x in intensit[i]]
        elif len(args.file) == 1 :
            newmax= max(intensit[i]) * args.cut[i]
            intensit[i] = [newmax if x >= newmax else x for x in intensit[i]]
        else:
            newmax= min(intensit[i]) * args.cut[i]
            intensit[i] = [newmax if x <= newmax else x for x in intensit[i]]
    ### end cut

    ### shift manually:
    if len(args.shift) < len(args.file):
        print('    Provided less or no shift values than files, therfore nothing is shifted!')
    else:
        for i in range(len(args.file)):
            ppm[i] = [item + args.shift[i] for item in ppm[i]]


    figure = plt.figure(figsize=(11.69,8.27))  ### A4 in inches
    figure.suptitle(args.title, fontsize=args.fontsize, y=0.93)

    ### create axes
    if args.startremove and args.endremove is not None:
        print('    Plotting spectrum in range of {} to {} ppm,'.format(args.start, args.end)) 
        print('    with data section between {} and {} ppm removed.'.format(args.startremove, args.endremove))
        #ratio:
        a = float(args.end) - float(args.endremove)
        b = float(args.startremove) - float(args.start)

# ontop and removed
        if args.ontop:
            if len(args.file) == 1:
                print('    Plotting ontop with only one file is useless! Going to exit!')
                exit(1)
            print('    Plotting ontop!')
            gs = gridspec.GridSpec(1,2,width_ratios=[a/b, 1])
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1])
            if len(args.file) == 2:
                max0 = max([yval for yval in intensit[0]])
                max0 = max0 + 0.000001 if max0 == 0 else max0
                max1 = min([yval for yval in intensit[1]])
                max1 = max1 + 0.000001 if max1 == 0 else max1

                ax1.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                ax1.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
                axdefaultsettings(ax1) ## own function
                ax1.set_xlim(left=args.end,right=args.endremove) ### sets x limits must be here
                ax1.tick_params(axis='x', bottom=True,labelbottom=True)
                ax1.spines['bottom'].set_visible(True)
                ax1.get_yaxis().set_visible(False)

                ax2.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                ax2.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
                axdefaultsettings(ax2)
                ax2.set_xlim(left=args.startremove,right=args.start) ### sets x limits must be here
                ax2.spines['bottom'].set_visible(True)
                ax2.tick_params(axis='x',bottom=True,labelbottom=True)
                ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

                ### make ticks equal
                ax1,ax2 = equal_ticks(ax1, ax2)

                # From https://matplotlib.org/examples/pylab_examples/broken_axis.html
                d = .015  # how big to make the diagonal lines in axes coordinates
                # arguments to pass to plot, just so we don't keep repeating them
                d2 = .015*(a/b)
                kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
                ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)
                kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)
                ax2.plot((- d2,  + d2), (-d, +d), **kwargs)
                #https://stackoverflow.com/questions/42045767/how-can-i-change-the-x-axis-in-matplotlib-so-there-is-no-white-space

            if len(args.file) == 3:
                max0 = max([yval for yval in intensit[0]])
                max0 = max0 + 0.000001 if max0 == 0 else max0
                max1 = max([yval for yval in intensit[1]])
                max1 = max1 + 0.000001 if max1 == 0 else max1
                max2 = min([yval for yval in intensit[2]])
                max2 = max2 + 0.000001 if max2 == 0 else max2

                ax1.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                ax1.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
                ax1.plot(ppm[2], [yval/max2 for yval in intensit[2]], label=args.label[2], color=args.colors[2], linewidth=args.linewidth)
                axdefaultsettings(ax1) ## own function
                ax1.autoscale(enable=False, axis='y')
                ax1.set_xlim(left=args.end,right=args.endremove) ### sets x limits must be here
                ax1.tick_params(axis='x', bottom=True,labelbottom=True)
                ax1.spines['bottom'].set_visible(True)
                ax1.get_yaxis().set_visible(False)

                ax2.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                ax2.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
                ax2.plot(ppm[2], [yval/max2 for yval in intensit[2]], label=args.label[2], color=args.colors[2], linewidth=args.linewidth)
                axdefaultsettings(ax2)
                ax2.autoscale(enable=False, axis='y')
                ax2.set_xlim(left=args.startremove,right=args.start) ### sets x limits must be here
                ax2.spines['bottom'].set_visible(True)
                ax2.tick_params(axis='x',bottom=True,labelbottom=True)
                ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

                ### make ticks equal
                ax1,ax2 = equal_ticks(ax1, ax2)
                ###
                # From https://matplotlib.org/examples/pylab_examples/broken_axis.html
                d = .015  # how big to make the diagonal lines in axes coordinates
                # arguments to pass to plot, just so we don't keep repeating them
                d2 = .015*(a/b)
                kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
                ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)  
                kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)  
                ax2.plot((- d2,  + d2), (-d, +d), **kwargs)  
                #https://stackoverflow.com/questions/42045767/how-can-i-change-the-x-axis-in-matplotlib-so-there-is-no-white-space
# removed and not ontop
        if len(args.file) == 1 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file), 2, width_ratios=[a/b, 1])
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1])

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1) ## own function 
            ax1.set_xlim(left=args.end,right=args.endremove) ### sets x limits must be here
            ax1.tick_params(axis='x', bottom=True,labelbottom=True)
            ax1.spines['bottom'].set_visible(True)
            ax1.get_yaxis().set_visible(False)
            ax2.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax2)
            ax2.set_xlim(left=args.startremove,right=args.start) ### sets x limits must be here
            ax2.spines['bottom'].set_visible(True)
            ax2.tick_params(axis='x',bottom=True,labelbottom=True)
            ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            ### make ticks equal
            ax1,ax2 = equal_ticks(ax1, ax2)

           # ### make ticks equal
           # xticksax1 = (abs(ax1.get_xticks()[0]-ax1.get_xticks()[1]))
           # xticksax2 = (abs(ax2.get_xticks()[0]-ax2.get_xticks()[1]))
           # if xticksax1 >= xticksax2:
           #     ax2.set_xticks(np.arange(args.start,args.startremove, step=xticksax1))
           # else:
           #     ax1.set_xticks(np.arange(args.endremove,args.end, step=xticksax2))

            #hi=True
            #if hi:
            #    xticksax1=0.5
            #    xticksax2=0.5
            #    ax2.set_xticks(np.arange(args.start,args.startremove, step=xticksax1))
            #    ax1.set_xticks(np.arange(args.endremove,args.end, step=xticksax2))


            ###
            # From https://matplotlib.org/examples/pylab_examples/broken_axis.html
            d = .015  # how big to make the diagonal lines in axes coordinates
            # arguments to pass to plot, just so we don't keep repeating them
            d2 = .015*(a/b)
            kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
            ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)  
            kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)  
            ax2.plot((- d2,  + d2), (-d, +d), **kwargs)  
            #https://stackoverflow.com/questions/42045767/how-can-i-change-the-x-axis-in-matplotlib-so-there-is-no-white-space
            #test
            ax1.spines['bottom'].set_position(('outward',1))
            ax2.spines['bottom'].set_position(('outward',1))
        if len(args.file) == 2 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file), 2, width_ratios=[a/b, 1]) 
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1])
            ax3 = plt.subplot(gs[2], sharex=ax1)
            ax4 = plt.subplot(gs[3], sharex=ax2)

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1) ## own function 
            ax1.set_xlim(left=args.end,right=args.endremove) ### sets x limits must be here

            ax2.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax2)
            ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)
            ax2.set_xlim(left=args.startremove,right=args.start) ### sets x limits must be here

            ### make ticks equal
            ax1,ax2 = equal_ticks(ax1, ax2)
            ###
            ax3.plot(ppm[1], intensit[1], color=args.colors[1], linewidth=args.linewidth)
            axdefaultsettings(ax3)
            ax3.tick_params(axis='x', bottom=True,labelbottom=True)
            ax3.spines['bottom'].set_visible(True)
            ax3.get_yaxis().set_visible(False)
            #ax3.legend(loc='best')

            ax4.plot(ppm[1], intensit[1], label= args.label[1], color=args.colors[1], linewidth=args.linewidth)
            #ax4.set_xlabel("$\delta$/ppm")
            axdefaultsettings(ax4)
            ax4.spines['bottom'].set_visible(True)
            ax4.tick_params(axis='x',bottom=True,labelbottom=True)
            ax4.legend(loc='lower right',frameon=args.keybox, fontsize=args.fontsize)

            # From https://matplotlib.org/examples/pylab_examples/broken_axis.html
            d = .015  # how big to make the diagonal lines in axes coordinates
            # arguments to pass to plot, just so we don't keep repeating them
            d2 = .015*(a/b)
            kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
            ax3.plot((1 - d, 1 + d), (-d, +d), **kwargs)  
            kwargs = dict(transform=ax3.transAxes, color='k', clip_on=False)
            ax3.plot((1 - d, 1 + d), (-d, +d), **kwargs) 
            kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)  
            ax2.plot((- d2,  + d2), (-d, +d), **kwargs)  
            kwargs = dict(transform=ax4.transAxes, color='k', clip_on=False)
            ax4.plot((- d2,  + d2), (-d, +d), **kwargs)

        if len(args.file) == 3 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file), 2, width_ratios=[a/b, 1]) 
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1])
            ax3 = plt.subplot(gs[2], sharex=ax1)
            ax4 = plt.subplot(gs[3], sharex=ax2)
            ax5 = plt.subplot(gs[4], sharex=ax1)
            ax6 = plt.subplot(gs[5], sharex=ax2)

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1)
            ax1.set_xlim(left=args.end,right=args.endremove) ### sets x limits must be here

            ax2.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax2)
            ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)
            ax2.set_xlim(left=args.startremove,right=args.start) ### sets x limits must be here

            ### make ticks equal
            ax1,ax2 = equal_ticks(ax1, ax2)
            ###
            ax3.plot(ppm[1], intensit[1], color=args.colors[1], linewidth=args.linewidth)
            axdefaultsettings(ax3)
            #ax3.legend(loc='best')

            ax4.plot(ppm[1], intensit[1], label= args.label[1], color=args.colors[1], linewidth=args.linewidth)
            axdefaultsettings(ax4)
            ax4.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            ax5.plot(ppm[2], intensit[2], color=args.colors[2], linewidth=args.linewidth)
            axdefaultsettings(ax5)
            ax5.tick_params(axis='x', bottom=True,labelbottom=True)
            ax5.spines['bottom'].set_visible(True)
            ax5.get_yaxis().set_visible(False)

            ax6.plot(ppm[2], intensit[2], label= args.label[2], color=args.colors[2], linewidth=args.linewidth)
            axdefaultsettings(ax6)
            ax6.tick_params(axis='x', bottom=True,labelbottom=True)
            ax6.spines['bottom'].set_visible(True)
            ax6.get_yaxis().set_visible(False)
            ax6.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            # From https://matplotlib.org/examples/pylab_examples/broken_axis.html
            d = .015  # how big to make the diagonal lines in axes coordinates
            # arguments to pass to plot, just so we don't keep repeating them
            d2 = .015*(a/b)
            kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
            ax3.plot((1 - d, 1 + d), (-d, +d), **kwargs)
            kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)
            ax2.plot((- d2,  + d2), (-d, +d), **kwargs)
            kwargs = dict(transform=ax4.transAxes, color='k', clip_on=False)
            ax4.plot((- d2,  + d2), (-d, +d), **kwargs)
            kwargs = dict(transform=ax3.transAxes, color='k', clip_on=False)
            ax5.plot((1- d, 1 + d), (-d, +d), **kwargs)
            kwargs = dict(transform=ax5.transAxes, color='k', clip_on=False)
            ax5.plot((1- d, 1 + d), (-d, +d), **kwargs)
            kwargs = dict(transform=ax6.transAxes, color='k', clip_on=False)
            ax6.plot((- d2,  + d2), (-d, +d), **kwargs)

    elif args.startremove is None or args.endremove is None:  #spectrum not removed!
        print('    Plotting complete spectrum in range of {} to {} ppm.'.format(args.start, args.end))
# spectra ontop of each other
        if args.ontop:
            if len(args.file) == 1:
                print('    Plotting ontop with only one file is useless! Going to exit!')
                exit(1)
            print('    Plotting ontop!')
            gs = gridspec.GridSpec(1,1)
            ax1 = plt.subplot(gs[0])
            if len(args.file) == 2:
                max0 = max([yval for yval in intensit[0]])
                max0 = max0 + 0.000001 if max0 == 0 else max0
                ax1.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                max1 = min([yval for yval in intensit[1]])
                max1 = max1 + 0.000001 if max1 == 0 else max1
                ax1.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
            if len(args.file) == 3:
                max0 = max([yval for yval in intensit[0]])
                max0 = max0 + 0.000001 if max0 == 0 else max0
                ax1.plot(ppm[0], [yval/max0 for yval in intensit[0]], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
                max1 = max([yval for yval in intensit[1]])
                max1 = max1 + 0.000001 if max1 == 0 else max1
                ax1.plot(ppm[1], [yval/max1 for yval in intensit[1]], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
                max2 = min([yval for yval in intensit[2]])
                max2 = max2 + 0.000001 if max2 == 0 else max2
                ax1.plot(ppm[2], [yval/max2 for yval in intensit[2]], label=args.label[2], color=args.colors[2], linewidth=args.linewidth)

            axdefaultsettings(ax1)
            ax1.autoscale(enable=False, axis='y')
            ax1.set_xlim(left=args.end,right=args.start) ### sets x limits must be here
            ax1.tick_params(axis='x', bottom=True,labelbottom=True)
            ax1.spines['bottom'].set_visible(True)
            ax1.get_yaxis().set_visible(False)
            ax1.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)
# spectra not removed and not ontop (normal case)
        elif len(args.file) == 1 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file),1)
            ax1 = plt.subplot(gs[0])

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1)
            ax1.set_xlim(left=args.end,right=args.start) ### sets x limits must be here
            ax1.tick_params(axis='x', bottom=True,labelbottom=True)
            ax1.spines['bottom'].set_visible(True)
            ax1.get_yaxis().set_visible(False)
            ax1.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)
            
            #new
            ax1.spines['bottom'].set_position(('outward',1)) # set spine (in picture the x axis down by x points)

        if len(args.file) == 2 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file),1)
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1], sharex=ax1)

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1)
            ax1.set_xlim(left=args.end,right=args.start) ### sets x limits must be here
            ax1.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            ax2.plot(ppm[1], intensit[1], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
            axdefaultsettings(ax2)
            ax2.tick_params(axis='x', bottom=True,labelbottom=True)
            ax2.spines['bottom'].set_visible(True)
            ax2.get_yaxis().set_visible(False)
            ax2.legend(loc='lower right',frameon=args.keybox, fontsize=args.fontsize)

        if len(args.file) ==3 and not args.ontop:
            gs = gridspec.GridSpec(len(args.file),1)
            ax1 = plt.subplot(gs[0])
            ax2 = plt.subplot(gs[1], sharex=ax1)
            ax3 = plt.subplot(gs[2], sharex=ax1)

            ax1.plot(ppm[0], intensit[0], label=args.label[0], color=args.colors[0], linewidth=args.linewidth)
            axdefaultsettings(ax1)
            ax1.set_xlim(left=args.end,right=args.start) ### sets x limits must be here
            ax1.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            ax2.plot(ppm[1], intensit[1], label=args.label[1], color=args.colors[1], linewidth=args.linewidth)
            axdefaultsettings(ax2)
            ax2.set_xlim(left=args.end,right=args.start) ### sets x limits must be here
            ax2.legend(loc='upper right',frameon=args.keybox, fontsize=args.fontsize)

            ax3.plot(ppm[2], intensit[2], label=args.label[2], color=args.colors[2], linewidth=args.linewidth)
            axdefaultsettings(ax3)
            ax3.tick_params(axis='x', bottom=True,labelbottom=True)
            ax3.spines['bottom'].set_visible(True)
            ax3.get_yaxis().set_visible(False)
            ax3.legend(loc='lower right',frameon=args.keybox, fontsize=args.fontsize)

    figure.subplots_adjust(wspace=0.07, hspace=0.009)
    figure.text(0.5, 0.04, "$\delta$/ppm", ha='center', fontsize=args.fontsize)
    plt.savefig(args.out+".pdf", dpi=300)
    plt.savefig(args.out+".png")
    #plt.show()
    print('    Plot is saved to {}.pdf and {}.png!'.format(args.out, args.out))
    print('    All Done!')

